<?php
/**
 * index.php用コンテナクラス
 *
 * PHP versions 5
 *
 * LICENSE: This source file is licensed under the terms of the GNU General Public License.
 *
 * @package    Magic3 Framework
 * @author     平田直毅(Naoki Hirata) <naoki@aplo.co.jp>
 * @copyright  Copyright 2006-2009 Magic3 Project.
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL License
 * @version    SVN: $Id: default_mainmenuWidgetContainer.php 1476 2009-01-27 08:30:13Z fishbone $
 * @link       http://www.magic3.org
 */
require_once($gEnvManager->getContainerPath() . '/baseWidgetContainer.php');
require_once($gEnvManager->getCurrentWidgetDbPath() . '/default_mainmenuDb.php');

class default_mainmenuWidgetContainer extends BaseWidgetContainer
{
	private $db;			// DB接続オブジェクト
	private $outputText;	// 作成したメニュー
	private $_menuType;	// メニュータイプ(0=テーブル、1=リスト)
	private $_menuTitle;	// メニュータイトル
	const MAIN_MENU_ID = 'main_menu';			// メインメニューID
	const LOGIN_MENU_ID = 'main_login_user';		// ログインユーザ向けのメインメニューID
	
	/**
	 * コンストラクタ
	 */
	function __construct()
	{
		// 親クラスを呼び出す
		parent::__construct();
		
		// DBオブジェクト作成
		$this->db = new default_mainmenuDb();
	}
	/**
	 * ウィジェット単位のアクセス制御
	 *
	 * @param RequestManager $request		HTTPリクエスト処理クラス
	 */
	function _checkAccess($request)
	{
		return true;
	}
	/**
	 * テンプレートファイルを設定
	 *
	 * _assign()でデータを埋め込むテンプレートファイルのファイル名を返す。
	 * 読み込むディレクトリは、「自ウィジェットディレクトリ/include/template」に固定。
	 *
	 * @param RequestManager $request		HTTPリクエスト処理クラス
	 * @param object         $param			任意使用パラメータ。そのまま_assign()に渡る
	 * @return string 						テンプレートファイル名。テンプレートライブラリを使用しない場合は空文字列「''」を返す。
	 */
	function _setTemplate($request, &$param)
	{
		// メニュー情報を取得
		// ログインメニューを表示するときでも、タイトルはメインメニューのタイトルを使用する
		$ret = $this->db->getMenu(self::MAIN_MENU_ID, $this->gEnv->getCurrentLanguage(), $row);
		if (!$ret){// 現在の言語で作成できない場合はデフォルト言語で作成
			$ret = $this->db->getMenu(self::MAIN_MENU_ID, $this->gEnv->getDefaultLanguage(), $row);
		}
		if ($ret){
			$this->_menuType = $row['me_type'];	// メニュータイプ
			if ($row['me_show_name']){		// メニュータイトル表示のとき
				$this->_menuTitle = $row['me_name'];
			}
		}
		// テンプレートファイル決定
		$filename = 'menu_v_';
		if ($this->_menuType == 0){			// テーブルタイプのとき
			$filename .= 'table.tmpl.html';
		} else if ($this->_menuType == 1){		// リストタイプのとき
			$filename .= 'list.tmpl.html';
		} else {
		}
		return $filename;
	}
	/**
	 * テンプレートにデータ埋め込む
	 *
	 * _setTemplate()で指定したテンプレートファイルにデータを埋め込む。
	 *
	 * @param RequestManager $request		HTTPリクエスト処理クラス
	 * @param object         $param			任意使用パラメータ。_setTemplate()と共有。
	 * @param								なし
	 */
	function _assign($request, &$param)
	{
		// ユーザのログイン状況から表示するメニューのタイプを決定する
		$menuType = self::MAIN_MENU_ID;
		if ($this->gEnv->isCurrentUserLogined()){		// ログイン中のとき
			// メニューデータが存在するかどうかチェック
			if ($this->db->isExistsMenuItems(self::LOGIN_MENU_ID)) $menuType = self::LOGIN_MENU_ID;
		}
		
		// メニューテーブルのパラメータ
		$this->tmpl->addVar("_widget", "default_menu_param", $this->gDesign->getDefaultWidgetTableParam());
		
		// メニュータイトルの設定
		if (!empty($this->_menuTitle)){
			$titleString = '';
			if ($this->_menuType == 0){			// テーブルタイプのとき
				$titleString .= '<tr><th>' . $this->_menuTitle . '</th><tr>';
			} else if ($this->_menuType == 1){		// リストタイプのとき
				$titleString .= '<lh>' . $this->_menuTitle . '</lh>';
			} else {
			}
			$this->tmpl->addVar("_widget", "title", $titleString);
		}
		
		// メインメニューの項目取得
		// 現在の言語で作成
		$this->db->getMenuItems(array($this, 'itemsLoop'), $menuType, $this->gEnv->getCurrentLanguage());
		if ($this->outputText == ''){// 現在の言語で作成できない場合はデフォルト言語で作成
			$this->db->getMenuItems(array($this, 'itemsLoop'), $menuType, $this->gEnv->getDefaultLanguage());
		}
	}
	/**
	 * 取得したメニュー項目をテンプレートに設定する
	 *
	 * @param int		$index			行番号
	 * @param array		$fetchedRow		取得行
	 * @param object	$param			任意使用パラメータ
	 * @return bool						trueを返すとループ続行。falseを返すとその時点で終了。
	 */
	function itemsLoop($index, $fetchedRow)
	{
		// 初期処理
		if ($index == 0){
//			$this->_menuType = $fetchedRow['mi_type'];	// メニュー項目のタイプ
		}

		// リンクタイプに合わせてタグを生成
		$option = '';
		switch ($fetchedRow['mi_link_type']){
			case 0:			// 同ウィンドウで開くリンク
				break;
			case 1:			// 別ウィンドウで開くリンク
				$option = 'target="_blank"';
				break;
		}
		
		// 「mainlevel」「sublevel」「active_menu」クラス名を設定する
		$name = '';
		if ($fetchedRow['mi_show_name']){	// 名前を表示するとき
			$name = $fetchedRow['mi_name'];
		}
		// リンク先の作成
		$linkUrl = $fetchedRow['mi_link_url'];
		$linkUrl = str_replace(M3_TAG_START . M3_TAG_MACRO_ROOT_URL . M3_TAG_END, $this->gEnv->getRootUrl(), $linkUrl);
		if ($fetchedRow['mi_enable']){		// 遷移可能なとき
			$link = '<a href="' . $this->convertUrlToHtmlEntity($linkUrl) . '" class="mainlevel" ' . $option . '>' . $name . '</a>';
		} else {
			$link = $name;
		}
		
		$row = array(
			'menu_class' => 'mainlevel',	//
			'link' => $link		// リンク
		);
		$this->tmpl->addVars('linklist', $row);
		$this->tmpl->parseTemplate('linklist', 'a');
		
		$this->outputText .= $fetchedRow['mi_name'];
		return true;
	}
}
?>
